# Generated by Django 3.1.6 on 2021-06-24 14:14

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_history', '0001_initial'),
    ]

    operations = [
        migrations.AlterField(
            model_name='action',
            name='ip',
            field=models.CharField(max_length=255, null=True, verbose_name='IP адрес'),
        ),
    ]
